@@ -1,7 +1,13 @@
+{-# LANGUAGE CPP #-}
+
+#if __GLASGOW_HASKELL__ >= 708
+{-# LANGUAGE DeriveDataTypeable #-}
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Bifunctor.Flip
--- Copyright   :  (C) 2008-2013 Edward Kmett,
+-- Copyright   :  (C) 2008-2015 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 --
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
@@ -13,20 +19,31 @@ module Data.Bifunctor.Flip
   ( Flip(..)
   ) where
 
+#if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
+#endif
+
 import Data.Biapplicative
-import Data.Bifunctor.Apply
 import Data.Bifoldable
 import Data.Bitraversable
+
+#if __GLASGOW_HASKELL__ < 710
 import Data.Foldable
 import Data.Monoid
-import Data.Semigroup.Bifoldable
-import Data.Semigroup.Bitraversable
 import Data.Traversable
+#endif
+
+#if __GLASGOW_HASKELL__ >= 708
+import Data.Typeable
+#endif
 
 -- | Make a 'Bifunctor' flipping the arguments of a 'Bifunctor'.
 newtype Flip p a b = Flip { runFlip :: p b a }
-  deriving (Eq,Ord,Show,Read)
+  deriving ( Eq, Ord, Show, Read
+#if __GLASGOW_HASKELL__ >= 708
+           , Typeable
+#endif
+           )
 
 instance Bifunctor p => Bifunctor (Flip p) where
   first f = Flip . second f . runFlip
@@ -47,10 +64,6 @@ instance Biapplicative p => Biapplicative (Flip p) where
   Flip fg <<*>> Flip xy = Flip (fg <<*>> xy)
   {-# INLINE (<<*>>) #-}
 
-instance Biapply p => Biapply (Flip p) where
-  Flip fg <<.>> Flip xy = Flip (fg <<.>> xy)
-  {-# INLINE (<<.>>) #-}
-
 instance Bifoldable p => Bifoldable (Flip p) where
   bifoldMap f g = bifoldMap g f . runFlip
   {-# INLINE bifoldMap #-}
@@ -66,11 +79,3 @@ instance Bitraversable p => Bitraversable (Flip p) where
 instance Bitraversable p => Traversable (Flip p a) where
   traverse f = fmap Flip . bitraverse f pure . runFlip
   {-# INLINE traverse #-}
-
-instance Bifoldable1 p => Bifoldable1 (Flip p) where
-  bifoldMap1 f g = bifoldMap1 g f . runFlip
-  {-# INLINE bifoldMap1 #-}
-
-instance Bitraversable1 p => Bitraversable1 (Flip p) where
-  bitraverse1 f g = fmap Flip . bitraverse1 g f . runFlip
-  {-# INLINE bitraverse1 #-}
